import { AES,enc } from 'crypto-js'

export interface USR<U> {
    app:string;
    timeStamp:number;
    userInfo?:U;
    token?:string | undefined;
    encode?:boolean;
    /** 1s,1m,1h,1d */
    expire?:string | undefined;
}

/**获取app相关的本地数据 */
const getStore =<T> (storeName:string) =>{
    const store =  localStorage.getItem(storeName)
    const time = new Date().getTime()
    if(!store){
        console.log('没有找到用户信息');
        return
    }
    let data = JSON.parse(store) as USR<T> 
    const expireDate = parseInt(data.expire?data.expire:'')
    if(data.token){
        data.token = data.encode?decodeToken(data.token,data.app):data.token
    }
    if(time>data.timeStamp + expireDate){
        localStorage.removeItem(data.app)
        return undefined
    }
    return data
}


const removeUser = (appname:string)=>{
    localStorage.removeItem(appname)
}

/**设置app相关的数据 */
const setStore =<T> (data:USR<T>) =>{
    const token = ecodeToken(data)
    const input = {
        app:data.app,
        timeStamp:new Date().getTime(),
        userInfo:data.userInfo?data.userInfo:{},
        token:token,
        encode:data.encode?data.encode:false,
        expire:data.expire?setExpire(data.expire):undefined
    }
    localStorage.setItem(input.app,JSON.stringify(input))
    return input
}

/**更新用户信息 */
const updateUserInfo=<T>(app:string, userInfo:T)=>{
    const store =  localStorage.getItem(app)
    if(store){
        const data = JSON.parse(store) as USR<T> 
        const newData = {
            app:data.app,
            timeStamp:data.timeStamp,
            userInfo:userInfo,
            token:data.token,
            encode:data.encode,
            /** 1s,1m,1h,1d */
            expire:data.expire
        }

        localStorage.setItem(app,JSON.stringify(newData))
    }else{
        console.log('没有找到用户信息');
        
    }

}


const ecodeToken = (data:USR<any>) =>{
    if(!data.token) return undefined
    if(!data.encode) return data.token
    const encodeToken =  AES.encrypt(data.token,data.app).toString()
    return encodeToken
}

const decodeToken = (token:string,secret:string) =>{
   const bytes =  AES.decrypt(token, secret);
   return bytes.toString(enc.Utf8)
}

const setExpire = (expireInput:string)=>{
    const t = expireInput.charAt(expireInput.length-1)
    const n = parseInt(expireInput.split(t)[0])
    const d = 1000  
    switch(t.toLowerCase()){
        case 's':
            return n*d
        case "m":
            return n*60*d
        case "h":
            return n*60*60*d
        case "d":
            return n*60*60*24*d
    }
}


export {setStore,getStore,updateUserInfo,removeUser}